module coeficientes

! objetivo: calcular os coeficientes das equaes discretizadas,
!           velocidades mdias e correes com plinha

use dados

implicit none

contains

!-------------------------------------------------

  subroutine ccu

    ! contorno esquerdo
    i = 1
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 1.0d0
       a(np,5) = 0.0d0
       b(np)   = 0.0d0
    end do

    ! contorno direito
    i = nx
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 1.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       b(np)   = 0.0d0
    end do

    ! contorno inferior
    j = 1
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 1.0d0
       b(np)   = 0.0d0
    end do

    ! contorno superior
    j = ny
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) = 1.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       if ( tipo == 1 ) then
          b(np) = 2.0d0*uex(np)
       else
          b(np) = 2.0d0*Ut
       end if
    end do
  
  end subroutine ccu

!-------------------------------------------------

  subroutine ccv

    ! contorno esquerdo
    i = 1
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 1.0d0
       a(np,5) = 0.0d0
       b(np)   = 0.0d0
    end do

    ! contorno direito
    i = nx
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 1.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       b(np)   = 0.0d0
    end do

    ! contorno inferior
    j = 1
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 1.0d0
       b(np)   = 0.0d0
    end do

    ! contorno superior
    j = ny
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) = 1.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       b(np)   = 0.0d0
    end do
  
  end subroutine ccv

!-------------------------------------------------

  subroutine ccp

    ! contorno esquerdo
    i = 1
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) =-1.0d0
       a(np,5) = 0.0d0
       b(np)   = pl(np+1) - pl(np+2)
       a(np+1,2) = 0.0d0
    end do

    ! contorno direito
    i = nx
    do j = 1, ny
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) =-1.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       b(np)   = pl(np-1) - pl(np-2)
       a(np-1,4) = 0.0d0
    end do

    ! contorno inferior
    j = 1
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) = 0.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) =-1.0d0
       b(np)   = pl(np+nx) - pl(np+2*nx)
       a(np+nx,1) = 0.0d0
    end do

    ! contorno superior
    j = ny
    do i = 2, nx-1
       np = (j-1)*nx + i
       a(np,1) =-1.0d0
       a(np,2) = 0.0d0
       a(np,3) = 1.0d0
       a(np,4) = 0.0d0
       a(np,5) = 0.0d0
       b(np)   = pl(np-nx) - pl(np-2*nx)
       a(np-nx,5) = 0.0d0
    end do
  
  end subroutine ccp

!-------------------------------------------------

  subroutine coeficientes_velocidade

    real*8 :: fme, fmw, fmn, fms ! fluxos de massa
    real*8 :: ae,  aw,  an,  as  ! coeficientes alfa do UDS

    ! volumes internos

    do i = 2, nx-1

       do j = 2, ny-1

          np = (j-1)*nx + i

          aw = dsign(5.0d-1,ue(np-1))
          ae = dsign(5.0d-1,ue(np))
          as = dsign(5.0d-1,vn(np-nx))
          an = dsign(5.0d-1,vn(np))

          fmw = ue(np-1)*dy
          fme = ue(np)*dy
          fms = vn(np-nx)*dx
          fmn = vn(np)*dx

          a(np,1) = -fms*(5.0d-1+as) - mi*dx/dy
          a(np,2) = -fmw*(5.0d-1+aw) - mi*dy/dx
          a(np,4) = -fme*(ae-5.0d-1) - mi*dy/dx
          a(np,5) = -fmn*(an-5.0d-1) - mi*dx/dy

          a(np,3) =  dx*dy/dt -(a(np,1)+a(np,2)+a(np,4)+a(np,5))

       end do

    end do

  end subroutine coeficientes_velocidade

!-------------------------------------------------

  subroutine fonte_velocidade_u

    real*8 :: fme, fmw, fmn, fms ! fluxos de massa
    real*8 :: ae,  aw,  an,  as  ! coeficientes alfa do UDS

    ! volumes internos

    do i = 2, nx-1

       do j = 2, ny-1

          np = (j-1)*nx + i

          aw = dsign(5.0d-1,ue(np-1))
          ae = dsign(5.0d-1,ue(np))
          as = dsign(5.0d-1,vn(np-nx))
          an = dsign(5.0d-1,vn(np))

          fmw = ue(np-1)*dy
          fme = ue(np)*dy
          fms = vn(np-nx)*dx
          fmn = vn(np)*dx

          cup(np) = fme*ae*(u(np)-u(np+1))  - fmw*aw*(u(np-1)-u(np)) &
                  + fmn*an*(u(np)-u(np+nx)) - fms*as*(u(np-nx)-u(np))

          b(np) = dx*dy*ua(np)/dt + cup(np)*beta &
                - 5.0d-1*(p(np+1)-p(np-1))*dy

       end do

    end do

  end subroutine fonte_velocidade_u

!-------------------------------------------------

  subroutine fonte_velocidade_v

    real*8 :: fme, fmw, fmn, fms ! fluxos de massa
    real*8 :: ae,  aw,  an,  as  ! coeficientes alfa do UDS

    ! volumes internos

    do i = 2, nx-1

       do j = 2, ny-1

          np = (j-1)*nx + i

          aw = dsign(5.0d-1,ue(np-1))
          ae = dsign(5.0d-1,ue(np))
          as = dsign(5.0d-1,vn(np-nx))
          an = dsign(5.0d-1,vn(np))

          fmw = ue(np-1)*dy
          fme = ue(np)*dy
          fms = vn(np-nx)*dx
          fmn = vn(np)*dx

          cvp(np) = fme*ae*(v(np)-v(np+1))  - fmw*aw*(v(np-1)-v(np)) &
                  + fmn*an*(v(np)-v(np+nx)) - fms*as*(v(np-nx)-v(np))

          b(np) = dx*dy*va(np)/dt + cvp(np)*beta &
                - 5.0d-1*(p(np+nx)-p(np-nx))*dx  &
                - bg(np)*dx*dy

       end do

    end do

  end subroutine fonte_velocidade_v

!-------------------------------------------------

  subroutine coeficientes_pressao

    real*8 :: fme, fmw, fmn, fms ! fluxos de massa

    ! volumes internos

    do i = 2, nx-1

       do j = 2, ny-1

          np = (j-1)*nx + i

          fmw = ue(np-1)*dy
          fme = ue(np)*dy
          fms = vn(np-nx)*dx
          fmn = vn(np)*dx

          a(np,1) = -dt*dx/dy
          a(np,2) = -dt*dy/dx
          a(np,4) = -dt*dy/dx
          a(np,5) = -dt*dx/dy

          a(np,3) = -(a(np,1)+a(np,2)+a(np,4)+a(np,5))

          b(np)   = fmw - fme + fms - fmn

       end do

    end do

  end subroutine coeficientes_pressao

!-------------------------------------------------

  subroutine media_coef_x_vel

    real*8  :: somaup, somaue, sapue, somavp, somavn, sapvn
    integer :: npw,npe,nps,npn,npne,npse,npee,npnw,npnn

    ! velocidades ue

    do i = 2, nx-2

       do j = 2, ny-1

          np = (j-1)*nx + i

          npw = np - 1
          npe = np + 1
          nps = np - nx
          npn = np + nx
          npne = npn + 1
          npse = nps + 1
          npee = npe + 1

          somaup=-(a(np,1)*u(nps) + a(np,2)*u(npw)  &
                 + a(np,4)*u(npe) + a(np,5)*u(npn) )

          somaue=-(a(npe,1)*u(npse) + a(npe,2)*u(np) &
                 + a(npe,4)*u(npee) + a(npe,5)*u(npne) )

          sapue = a(np,3) + a(npe,3)

          ue(np) = (somaup + somaue + 2.0d0*dx*dy*uea(np)/dt &
                 + (cup(np)+cup(np+1))*beta) / sapue         &
                 -  2.0d0 * DY*(P(NPE)-P(NP)) / sapue

       end do

    end do

    ! velocidades vn

    do i = 2, nx-1

       do j = 2, ny-2

          np = (j-1)*nx + i

          npw = np - 1
          npe = np + 1
          nps = np - nx
          npn = np + nx
          npne = npn + 1
          npnw = npn - 1
          npnn = npn + nx

          somavp=-(a(np,1)*v(nps) + a(np,2)*v(npw)   &
                + a(np,4)*v(npe) + a(np,5)*v(npn) )

          somavn=-(a(npn,1)*v(np)   + a(npn,2)*v(npnw)  &
                + a(npn,4)*v(npne) + a(npn,5)*v(npnn) )

          sapvn = a(np,3) + a(npn,3)
  
          vn(np) = (somavp + somavn + 2.0d0*dx*dy*vna(np)/dt &
                 + (cvp(np)+cvp(np+nx))*beta) / sapvn        &
                 -  2.0d0 * DX*(P(NPN)-P(NP)) / sapvn		 &
                 - (bg(np) + bg(np+nx))*dx*dy / sapvn

       end do

    end do

  end subroutine media_coef_x_vel

!-------------------------------------------------

  subroutine atualiza_pressao

    ! volumes internos

    p = p + pl

    ! contorno esquerdo
    i = 1
    do j = 1, ny
       np = (j-1)*nx + i
       p(np) = 2.0d0*p(np+1) - p(np+2)
    end do

    ! contorno direito
    i = nx
    do j = 1, ny
       np = (j-1)*nx + i
       p(np) = 2.0d0*p(np-1) - p(np-2)
    end do

    ! contorno inferior
    j = 1
    do i = 2, nx-1
       np = (j-1)*nx + i
       p(np) = 2.0d0*p(np+nx) - p(np+2*nx)
    end do

    ! contorno superior
    j = ny
    do i = 2, nx-1
       np = (j-1)*nx + i
       p(np) = 2.0d0*p(np-nx) - p(np-2*nx)
    end do
  
  end subroutine atualiza_pressao

!-------------------------------------------------

  subroutine atualiza_velocidades_nodais

    ! volumes internos

    do i = 2, nx-1
       do j = 2, ny-1
          np = (j-1)*nx + i
          u(np) = u(np) - 5.0d-1*(pl(np+1)-pl(np-1))*dt/dx
          v(np) = v(np) - 5.0d-1*(pl(np+nx)-pl(np-nx))*dt/dy
       end do
    end do

    ! contorno esquerdo
    i = 1
    do j = 1, ny
       np = (j-1)*nx + i
       u(np) = -u(np+1)
       v(np) = -v(np+1)
    end do

    ! contorno direito
    i = nx
    do j = 1, ny
       np = (j-1)*nx + i
       u(np) = -u(np-1)
       v(np) = -v(np-1)
    end do

    ! contorno inferior
    j = 1
    do i = 2, nx-1
       np = (j-1)*nx + i
       u(np) = -u(np+nx)
       v(np) = -v(np+nx)
    end do

    ! contorno superior
    j = ny
    do i = 2, nx-1
       np = (j-1)*nx + i
       if ( tipo == 1 ) then
          u(np) = 2.0d0*uex(np) - u(np-nx)
       else
          u(np) = 2.0d0*Ut - u(np-nx)
       end if
       v(np) = -v(np-nx)
    end do
  
  end subroutine atualiza_velocidades_nodais

!-------------------------------------------------

  subroutine atualiza_velocidades_medias

    ! velocidades ue

    do i = 2, nx-2
       do j = 2, ny-1
          np = (j-1)*nx + i
          ue(np) = ue(np) - (pl(np+1)-pl(np))*dt/dx
       end do
    end do

    ! velocidades vn

    do i = 2, nx-1
       do j = 2, ny-2
          np = (j-1)*nx + i
          vn(np) = vn(np) - (pl(np+nx)-pl(np))*dt/dy
       end do
    end do

  end subroutine atualiza_velocidades_medias

!-------------------------------------------------

end module coeficientes
