program STOKES_1p5

!     Escoamento Bidimensional Isotrmico e
!     Permanente de Fluido Incompressvel

!     Verso atual    1.5 (21 Out 07)

!     Verso          1.3 ( 6 Out 05)
!     Verso          1.2 (29 Jul 05)
!     Verso          1.1 (30 Abr 99)
!     Verso original 1.0 (25 Mar 99)

!     ltima alterao =    5 Nov 07

!     Autor: Carlos Henrique Marchi
!            DEMEC/UFPR, Curitiba, PR


! MODELO MATEMTICO (equaes diferenciais):
! -----------------------------------------

!   massa: du/dx + dv/dy = 0

!   QMLx:  d(u2)/dx + d(uv)/dy = -dp/dx + mi*laplaciano(u)

!   QMLy:  d(uv)/dx + d(v2)/dy = -dp/dy + mi*laplaciano(v) - S

!   x, y = coordenadas espaciais (m)
!   u, v = velocidades nas direes x e y (m/s)
!   p    = presso esttica (Pa)
!   mi   = viscosidade absoluta (Pa.s)
!   S    = termo fonte; nulo no problema de Ghia
!   Re   = Ut * L / mi (nmero de Reynolds)
!   Ut   = velocidade mxima da tampa da cavidade (1 m/s)
!   Cavidade quadrada (L=1 por H=1 m)
!   ro   = massa especfica (1 kg/m3)
!   Condio de contorno de Dirichlet para u e v
!   Tipos de problemas pr-definidos: 
!     1 = Shih: u na tampa  uma funo de x; u e v nulos nas outras paredes
!     2 = Ghia: u na tampa  constante = 1; u e v nulos nas outras paredes


! MODELO NUMRICO (resumo):
! ------------------------

!   Incgnitas: u, v, p
!   Condies de contorno de Dirichlet para u e v aplicadas com volumes fictcios
!   A p  extrapolado linearmente dos ns internos para os contornos
!   Mtodo numrico: volumes finitos
!   Funes de interpolao para u e v: 
!     CDS para os termos difusivos e de presso
!     UDS ou CDS (com correo adiada) para os termos advectivos
!   Malha uniforme em cada direo
!   Mtodo de acoplamento presso-velocidade: SIMPLEC
!   Soluo segregada (seqencial) das equaes diferenciais
!   Solvers: Jacobi, Gauss-Seidel, SOR, ADI e MSI-5
!   Preciso: dupla
!   Tempo  usado para avano do processo iterativo
!   Arranjo co-localizado de variveis (mdia NHT/1994)
!   Linguagem FORTRAN 95
!   Aplicativo usado: Fortran Intel 9.1
!   Tipo de projeto: Console; verso Release
!   Variveis de interesse:
!     Valor mnimo da funo de corrente, e seu X e Y, baseado na velocidade mdia
!        u na face leste de cada volume e com integral retngulo
!     Valor mnimo da funo de corrente, e seu X e Y, baseado na velocidade mdia
!        u na face leste de cada volume e com integral trapzio
!     Fluxo de massa que escoa em y=0.5 e x[0;0.5]: obtido com a integrao 
!        pela regra do retngulo da velocidade mdia v na face norte de cada volume
!     Idem com a regra do trapzio
!     Fora viscosa da placa sobre o fluido: obtida com integrao retngulo 
!        da derivada du/dy em y=1 obtida com velocidade nodal u e com UDS
!     Fora viscosa da placa sobre o fluido: obtida com integrao retngulo 
!        da derivada du/dy em y=1 obtida com velocidade nodal u e com UDS-2
!     Fora viscosa da placa sobre o fluido: obtida com integrao trapzio
!        da derivada du/dy em y=1 obtida com velocidade na face ue e com UDS
!     Fora viscosa da placa sobre o fluido: obtida com integrao retngulo 
!        da derivada du/dy em y=0 obtida com velocidade nodal u e com DDS
!     Derivada du/dy em x=0.5 e y=1: obtida com velocidade mdia u 
!        e com UDS; vlido s para nmero de volumes pares em x;
!     Presso no centro do domnio: obtido por mdia aritmtica de p nos
!        quatro ns vizinhos
!     Perfil da velocidade u em x = 0.5: obtido por mdia aritmtica das
!        velocidades na face leste nos dois volumes vizinhos
!     Perfil da velocidade v em y = 0.5: obtido por mdia aritmtica das
!        velocidades na face norte nos dois volumes vizinhos
!     Valor mnimo de u, e seu Y, do perfil da velocidade u em x = 0.5 obtido por
!        mdia aritmtica das velocidades na face leste nos dois volumes vizinhos
!     Valor mnimo de v, e seu X, do perfil da velocidade v em y = 0.5 obtido por
!        mdia aritmtica das velocidades na face norte nos dois volumes vizinhos
!     Valor mximo de v, e seu X, do perfil da velocidade v em y = 0.5 obtido por
!        mdia aritmtica das velocidades na face norte nos dois volumes vizinhos
!     Presso no centro do domnio obtido por mdia aritmtica de 4 p nodais
!   Estimativa inicial da soluo numrica  igual a:
!     Soluo analtica do problema de Shih ou Zero
!   Expresso genrica dos sistemas de equaes discretizados:
!     a(NP,1)*T(NPS) + a(NP,2)*T(NPW) + a(NP,3)*T(NP) + a(NP,4)*T(NPE) 
!                    + a(NP,5)*T(NPN) = b(NP)
!     onde T representa u, v ou plinha


! ARQUIVOS envolvidos no programa:
! -------------------------------

!   Executvel:
!     Stokes_1p5.exe

!   Dados:
!     Stokes_1p5.ent = arquivo principal de dados (do usurio) do programa 
!     Stokes_1p5_u_Shih.gnu = dados para gerar grfico dos perfis de u no problema de Shih
!     Stokes_1p5_v_Shih.gnu = dados para gerar grfico dos perfis de v no problema de Shih
!     Stokes_1p5_u_Ghia.gnu = dados para gerar grfico dos perfis de u no problema de Ghia
!     Stokes_1p5_v_Ghia.gnu = dados para gerar grfico dos perfis de v no problema de Ghia
!     Stokes_1p5_R.gnu      = dados para gerar grfico dos resduo das equaes ao longo das iteraes
!     Ghia_Re_100_U.dat   = perfil de u no problema de Ghia para Re = 100
!     Ghia_Re_1000_U.dat  = perfil de u no problema de Ghia para Re = 1000
!     Ghia_Re_10000_U.dat = perfil de u no problema de Ghia para Re = 10000
!     Ghia_Re_100_V.dat   = perfil de v no problema de Ghia para Re = 100
!     Ghia_Re_1000_V.dat  = perfil de v no problema de Ghia para Re = 1000
!     Ghia_Re_10000_V.dat = perfil de v no problema de Ghia para Re = 10000

!   Sadas:
!     "caso".txt = sada principal do programa
!     "caso".itn = algumas variveis de interesse ao longo das iteraes
!     "caso".Richrdson_3p0 = variveis de interesse para o programa Richardson 3.0
!     r.dat      = resduo das equaes ao longo das iteraes
!     ucex.dat   = perfil de u em x=1/2 (exato)
!     vcex.dat   = perfil de v em y=1/2 (exato).
!     u.dat      = perfil de u em x=1/2 (numrico)
!     v.dat      = perfil de v em y=1/2 (numrico)

!   Fontes:
!     Stokes_1p5_main.f90         = programa principal
!     Stokes_1p5_coeficientes.f90 = calcula coeficientes dos sistemas de equaes
!     Stokes_1p5_dados.f90        = l e lista os dados do programa
!     Stokes_1p5_resultados.f90   = resolve equaes e gera listagem dos resultados
!     Stokes_1p5_solvers.f90      = solvers disponveis
!     Stokes_1p5_variaveis.f90    = define todas as variveis globais do programa
!
!   Outros arquivos:
!     Notepad.exe    = editor de arquivos
!     Wgnuplot.exe   = programa gerador de grfico (verso 3.5)

! -----------------------------------------------

use resultados

! -----------------------------------------------

implicit none

! -----------------------------------------------

  call le_dados

  open(10,file=trim(adjustl(caso))//'.txt')

  call data_hora

  call mostra_dados

! -----------------------------------------------

  call solucao_numerica

  close (10)

  note_caso = 'notepad '//caso
  dos = system(note_caso) ! lista arquivo de resultados

! -----------------------------------------------

end
