module variaveis

! objetivo: definir as variveis globais do programa

use portlib

implicit none

integer :: nx  ! nmero total de volumes na direo X (com 2 fictcios)
integer :: ny  ! nmero total de volumes na direo Y (com 2 fictcios)
integer :: nxy ! nx * ny

integer :: i   ! nmero do volume na direo X
integer :: j   ! nmero do volume na direo Y
integer :: np  ! nmero do volume na direes X e Y combinados

integer :: solver ! 1=Jacobi; 2=Gauss-Seidel; 3=SOR; 4=ADI; 5=MSI
integer :: it     ! nmero da iterao do processo iterativo
integer :: im     ! nmero da iterao do ciclo da massa
integer :: itmax  ! nmero mximo de iteraes externas
integer :: imax   ! nmero mximo de iteraes no ciclo da massa
integer :: nitm_u ! nmero mximo de iteraes internas para u,v
real*8  :: tolu   ! tolerncia do MSI para u,v
integer :: nitm_p ! nmero mximo de iteraes internas para pl
real*8  :: tolp   ! tolerncia do MSI para pl
real*8  :: w      ! coeficiente de relaxao do SOR
real*8  :: tole   ! tolerncia sobre as iteraes externas (norma L1)
real*8  :: norma  ! norma L1 de todos os sistemas lineares
integer :: w_cam  ! =1, escreve campos;   =0, no
integer :: w_r    ! freqncia de escrita dos resduos
integer :: tipo   ! =1, problema de Shih;   =2, problema de Ghia
integer :: base   ! malha-base para u e v de interesse
integer :: ci     ! tipo da condio inicial: 0=nulo; 1=soluo analtica de Shih

real*8  :: dx     ! distncia entre ns na direo X
real*8  :: dy     ! distncia entre ns na direo Y

real*8  :: dt     ! incremento de tempo
real*8  :: L      ! comprimento do domnio na direo X
real*8  :: H      ! comprimento do domnio na direo Y
real*8  :: Re     ! nmero de Reynolds
real*8  :: mi     ! viscosidade absoluta
real*8  :: Ut     ! velocidade da tampa

! variveis de interesse
real*8  :: uc     ! velocidade u no centro do domnio (mdia de ue)
real*8  :: umin   ! ue mnimo em x=1/2
real*8  :: y_umin ! y de umin
real*8  :: vc     ! velocidade v no centro do domnio (mdia de vn)
real*8  :: vmin   ! vn mnimo em y=1/2
real*8  :: x_vmin ! x de vmin
real*8  :: vmax   ! vn mximo em y=1/2
real*8  :: x_vmax ! x de vmax
real*8  :: pc     ! presso no centro do domnio (mdia de p)
real*8  :: fmr    ! fluxo de massa em y=1/2 e x[0;1/2] com vn e integral retngulo
real*8  :: fmt    ! fluxo de massa em y=1/2 e x[0;1/2] com vn e integral trapzio
real*8  :: dudy   ! derivada du/dy com ue e UDS em x=0.5 e y=1 (1/s); vlido para nx par 
real*8  :: Fsr    ! fora viscosa na direo x da parede do contorno sul sobre o
                  ! fluido com DDS, uP e integral retngulo 
real*8  :: Fnr    ! fora viscosa na direo x da tampa (contorno norte) sobre o
                  ! fluido com UDS, uP e integral retngulo 
real*8  :: Fnr2   ! fora viscosa na direo x da tampa (contorno norte) sobre o
                  ! fluido com UDS-2, uP e integral retngulo 
real*8  :: Fnt    ! fora viscosa na direo x da tampa (contorno norte) sobre o
                  ! fluido com UDS, ue e integral trapzio 
real*8  :: psi_r_min  ! valor mnimo da funo de corrente resolvida com retngulo
real*8  :: xmin_psi_r ! x de psi_r_min
real*8  :: ymin_psi_r ! y de psi_r_min
real*8  :: psi_t_min  ! valor mnimo da funo de corrente resolvida com trapzio
real*8  :: xmin_psi_t ! x de psi_r_min
real*8  :: ymin_psi_t ! y de psi_r_min

real*8  :: beta   ! fator da funo de interpolao; 0=UDS; 1=CDS

logical :: dos    ! acessa prompt dos

character*20 :: caso  ! nome do arquivo de sada
character*50 :: title ! ttulo do grfico
character*62 :: head  ! ttulo do grfico + dia

real*8,dimension(:),allocatable   :: uex ! velocidade na direo X exato
real*8,dimension(:),allocatable   :: vex ! velocidade na direo Y exato
real*8,dimension(:),allocatable   :: pex ! presso exato

real*8,dimension(:),allocatable   :: bg  ! fonte exato de v

real*8,dimension(:),allocatable   :: u   ! velocidade na direo X
real*8,dimension(:),allocatable   :: v   ! velocidade na direo Y
real*8,dimension(:),allocatable   :: p   ! presso

real*8,dimension(:),allocatable   :: pl  ! variao da presso
real*8,dimension(:),allocatable   :: ue  ! velocidade na face leste na direo X
real*8,dimension(:),allocatable   :: vn  ! velocidade na face norte na direo Y

real*8,dimension(:),allocatable   :: cup ! fator da correo adiada u
real*8,dimension(:),allocatable   :: cvp ! fator da correo adiada v

real*8,dimension(:),allocatable   :: ua  ! u do instante anterior
real*8,dimension(:),allocatable   :: va  ! v do instante anterior
real*8,dimension(:),allocatable   :: uea ! ue do instante anterior
real*8,dimension(:),allocatable   :: vna ! vn do instante anterior

real*8,dimension(:,:),allocatable :: a  ! coeficientes de u, v, p

real*8,dimension(:,:),allocatable :: dl ! coeficientes de t low
real*8,dimension(:,:),allocatable :: du ! coeficientes de t up

real*8,dimension(:),allocatable   :: b  ! fontes de u, v, p

real*8,dimension(:),allocatable   :: psi_r ! funo de corrente com integral retngulo
real*8,dimension(:),allocatable   :: psi_t ! funo de corrente com integral trapzio

! a(NP,1)*T(NPS) + a(NP,2)*T(NPW) + a(NP,3)*T(NP) + a(NP,4)*T(NPE) + a(NP,5)*T(NPN) = b(NP)

! primeiro ndice = nmero do volume em X-Y
! segundo  ndice = conexo com volume vizinho;
!                   1=S; 2=W; 3=P; 4=E; 5=N

character*12 :: dia    ! data da simulao
character*8  :: hora   ! horrio da simulao
character*70 :: note_caso ! notepad + caso

real*8  :: norma_1 ! norma da primeira iterao
real*8  :: tcpu1   ! tempo de CPU inicial
real*8  :: tcpu2   ! tempo de CPU final
real*8  :: tcpu    ! tempo de CPU total acumulado
real*8  :: tcpuo   ! tempo de CPU de reincio
integer :: ito     ! nmero da iterao inicial no tempo de reincio

end